import produce from "immer";
import { catchError, debounceTime, from, map, mapTo, merge, of, Subject, take, takeUntil, tap } from "rxjs";
import { ModuleUtils } from "../../../utils/lang";
import { getTypeOperation, retryProcessing } from "../../common/utils";
export const initialStateAsyncCall = {
  params: {},
  status: {},
  errors: []
};
export class AsyncCall {
  constructor(params$, executeFn$, config, destroy$, setState) {
    this.params$ = params$;
    this.executeFn$ = executeFn$;
    this.config = config;
    this.destroy$ = destroy$;
    this.setState = setState;
    this.execute$ = new Subject();
    this.newState = initialStateAsyncCall;
    this.setData$Subject = new Subject();
    this.destroy$ = destroy$ ? destroy$ : new Subject();
    this.init = this.init.bind(this);
    this.getState = this.getState.bind(this);
    this.reload = this.reload.bind(this);
    this.initSetData = this.initSetData.bind(this);
    this.getStatus = this.getStatus.bind(this);
  }

  getStatus() {
    return this.newState.status.status;
  }

  initSetData() {
    this.setData$Subject.pipe(takeUntil(this.destroy$), tap(setData => {
      this.setState(prevState => {
        const newState = produce(draft => {
          const state = this.getState();
          debugger;
          draft.status.data = setData(state.status.data);
        })(prevState);
        this.newState = newState;
        return newState;
      });
    })).subscribe();
  }

  init() {
    this.initSetData();
    const processingStatus$ = new Subject();
    const params$ = this.params$ ? merge(this.params$, this.execute$) : this.execute$;
    const config = this.config ? this.config : {};
    const asyncOperation = getTypeOperation(config.processingType);
    merge(processingStatus$.pipe(mapTo({
      status: 'PROCESSING'
    })), params$.pipe(takeUntil(this.destroy$), debounceTime(10), tap(params => {
      this.setState(prevState => {
        const newState = produce(draft => {
          draft.params = params;
        })(prevState);
        this.newState = newState;
        return newState;
      });
      processingStatus$.next();
    }), asyncOperation(params => {
      const data$ = this.executeFn$ ? this.executeFn$(params) : of(params);
      const loadData$ = ModuleUtils.isObservable(data$) ? data$ : ModuleUtils.isPromise(data$) ? from(data$) : of(data$);
      const retryConfig = config.retry ? config.retry : {
        interval: 100
      };
      return loadData$.pipe(retryProcessing(retryConfig), take(1), map(data => {
        return {
          status: 'SUCCESS',
          data
        };
      }), catchError(error => {
        return of({
          status: 'ERROR',
          error
        });
      }));
    }))).pipe(takeUntil(this.destroy$), tap(processingStatus => {
      this.setState(prevState => {
        const newState = produce(draft => {
          draft.status.data = processingStatus.data;
          draft.status.error = processingStatus.error;
        })(prevState);
        this.newState = newState;
        return newState;
      });
    }), tap(processingStatus => {
      const params = this.newState.params;
      processingStatus.status === 'PROCESSING' && config.onProcessing && config.onProcessing(params);
      processingStatus.status === 'ERROR' && config.onError && config.onError(processingStatus.error, params);
      processingStatus.status === 'SUCCESS' && config.onSuccess && config.onSuccess(processingStatus.data, params);
      this.setState(prevState => {
        const newState = produce(draft => {
          draft.status.status = processingStatus.status;
        })(prevState);
        this.newState = newState;
        return newState;
      });
    })).subscribe();
  }

  getState() {
    return this.newState;
  }

  reload() {
    this.execute$.next(this.newState.params);
  }

  get execute() {
    return this.execute$.next.bind(this.execute$);
  }

  get setData() {
    return this.setData$Subject.next.bind(this.setData$Subject);
  }

}